@@ -1,8 +1,8 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
-# __coconut_hash__ = 0x1e9c5d2c
+# __coconut_hash__ = 0xebd80329
 
-# Compiled with Coconut version 1.5.0-post_dev57 [Fish License]
+# Compiled with Coconut version 1.5.0-post_dev74 [Fish License]
 
 """
 The mixture backend. Lets you specify a distribution over different possible algorithms.
@@ -27,24 +27,29 @@ if _coconut_module_name and _coconut_module_name[0].isalpha() and all(c.isalpha(
             try:
                 _coconut_v.__module__ = _coconut_full_module_name
             except AttributeError:
-                type(_coconut_v).__module__ = _coconut_full_module_name
+                _coconut_v_type = type(_coconut_v)
+                if getattr(_coconut_v_type, "__module__", None) == str("__coconut__"):
+                    _coconut_v_type.__module__ = _coconut_full_module_name
     _coconut_sys.modules[_coconut_full_module_name] = _coconut__coconut__
 from __coconut__ import *
 from __coconut__ import _coconut_call_set_names, _coconut, _coconut_MatchError, _coconut_igetitem, _coconut_base_compose, _coconut_forward_compose, _coconut_back_compose, _coconut_forward_star_compose, _coconut_back_star_compose, _coconut_forward_dubstar_compose, _coconut_back_dubstar_compose, _coconut_pipe, _coconut_star_pipe, _coconut_dubstar_pipe, _coconut_back_pipe, _coconut_back_star_pipe, _coconut_back_dubstar_pipe, _coconut_none_pipe, _coconut_none_star_pipe, _coconut_none_dubstar_pipe, _coconut_bool_and, _coconut_bool_or, _coconut_none_coalesce, _coconut_minus, _coconut_map, _coconut_partial, _coconut_get_function_match_error, _coconut_base_pattern_func, _coconut_addpattern, _coconut_sentinel, _coconut_assert, _coconut_mark_as_match, _coconut_reiterable
 _coconut_sys.path.pop(0)
+
 # Compiled Coconut: -----------------------------------------------------------
 
 
 
-import random
-
 from bbopt import constants
 from bbopt.util import convert_match_errors
 from bbopt.registry import alg_registry
 from bbopt.backends.util import Backend
 from bbopt.backends.util import get_backend
+from bbopt.backends.util import get_cum_probs_for
+from bbopt.backends.util import random_from_cum_probs
 
 
+# Backend:
+
 class MixtureBackend(Backend):
     """Mixture backend. Takes in a distribution over different possible algorithms
     of the form [(algorithm, weight)]. The properties selected_alg and selected_backend
@@ -54,9 +59,6 @@ class MixtureBackend(Backend):
     request_backend_store = True
     remove_erroring_algs = None
 
-    def __init__(self, examples, params, *args, **kwargs):
-        self.attempt_update(examples, params, *args, **kwargs)
-
     @override
     @convert_match_errors
     @_coconut_mark_as_match
@@ -70,24 +72,24 @@ class MixtureBackend(Backend):
             _coconut_match_temp_2 = _coconut_match_args[2] if _coconut.len(_coconut_match_args) > 2 else _coconut_match_kwargs.pop("params")
             _coconut_match_temp_3 = _coconut_match_args[3] if _coconut.len(_coconut_match_args) > 3 else _coconut_match_kwargs.pop("distribution")
             _coconut_match_temp_4 = _coconut_match_args[4] if _coconut.len(_coconut_match_args) > 4 else _coconut_match_kwargs.pop("remove_erroring_algs") if "remove_erroring_algs" in _coconut_match_kwargs else False
-            _coconut_match_temp_5 = _coconut_match_kwargs.pop("backend_store") if "backend_store" in _coconut_match_kwargs else _coconut_sentinel
+            _coconut_match_temp_5 = _coconut_match_kwargs.pop("_backend_store") if "_backend_store" in _coconut_match_kwargs else _coconut_sentinel
             if (_coconut_match_temp_5 is not _coconut_sentinel) and (not _coconut_match_kwargs):
                 self = _coconut_match_temp_0
                 examples = _coconut_match_temp_1
                 params = _coconut_match_temp_2
                 distribution = _coconut_match_temp_3
                 remove_erroring_algs = _coconut_match_temp_4
-                backend_store = _coconut_match_temp_5
+                _backend_store = _coconut_match_temp_5
                 _coconut_match_check_0 = True
         if not _coconut_match_check_0:
-            raise _coconut_FunctionMatchError('match def attempt_update(self, examples, params, distribution, remove_erroring_algs=False, *, backend_store):', _coconut_match_args)
+            raise _coconut_FunctionMatchError('match def attempt_update(self, examples, params, distribution, remove_erroring_algs=False, *, _backend_store):', _coconut_match_args)
 
         self.use_distribution(distribution, force=remove_erroring_algs != self.remove_erroring_algs)
 
         self.examples = examples
         self.params = params
         self.remove_erroring_algs = remove_erroring_algs
-        self.backend_store = backend_store
+        self.backend_store = _backend_store
 
         self.select_new_backend()
         return True
@@ -100,33 +102,13 @@ class MixtureBackend(Backend):
             distribution = tuple(distribution)
 
         if force or distribution != self.distribution:
-            self.set_cum_probs_for(distribution)
+            self.cum_probs = get_cum_probs_for(distribution)
             self.distribution = distribution
 
-    def set_cum_probs_for(self, distribution):
-        """Set the cum_probs used to pick the backend according to the given distribution."""
-        self.cum_probs = []
-        total_weight = sum((weight for alg, weight in distribution))
-        prev_cutoff = 0
-        for alg, weight in distribution:
-            if weight == float("inf"):
-                cutoff = 1
-            elif weight in (float("-inf"), float("nan")) or total_weight == float("nan"):
-                cutoff = prev_cutoff
-            else:
-                cutoff = prev_cutoff + weight / total_weight
-            self.cum_probs.append((alg, cutoff))
-            prev_cutoff = cutoff
-
     def select_new_backend(self):
         """Randomly select a new backend."""
 # randomly select algorithm
-        rand_val = random.random()
-        self.selected_alg = None
-        for alg, cutoff in self.cum_probs:
-            if rand_val <= cutoff:
-                self.selected_alg = alg
-                break
+        self.selected_alg = random_from_cum_probs(self.cum_probs)
         if self.selected_alg is None:
             raise ValueError("could not select backend from distribution: {_coconut_format_0}".format(_coconut_format_0=(self.distribution)))
 
@@ -145,7 +127,7 @@ class MixtureBackend(Backend):
         for alg, weight in self.distribution:
             if alg != self.selected_alg:
                 new_distribution.append((alg, weight))
-        self.set_cum_probs_for(new_distribution)
+        self.cum_probs = get_cum_probs_for(new_distribution)
         self.select_new_backend()
 
     @override
@@ -179,3 +161,7 @@ MixtureBackend.register_safe_alg_for("gaussian_process")
 MixtureBackend.register_safe_alg_for("random_forest")
 MixtureBackend.register_safe_alg_for("extra_trees")
 MixtureBackend.register_safe_alg_for("gradient_boosted_regression_trees")
+
+# we register meta alg mixtures here
+MixtureBackend.register_meta("tpe_or_gp", ("tree_structured_parzen_estimator", "safe_gaussian_process",))
+MixtureBackend.register_meta("any_fast", ("tree_structured_parzen_estimator", "safe_random_forest", "safe_extra_trees", "safe_gradient_boosted_regression_trees",))
