@@ -1,8 +1,8 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
-# __coconut_hash__ = 0x477f6619
+# __coconut_hash__ = 0x5e7343e6
 
-# Compiled with Coconut version 1.5.0-post_dev57 [Fish License]
+# Compiled with Coconut version 1.5.0-post_dev74 [Fish License]
 
 """
 Utilities for use in BBopt backends.
@@ -27,24 +27,32 @@ if _coconut_module_name and _coconut_module_name[0].isalpha() and all(c.isalpha(
             try:
                 _coconut_v.__module__ = _coconut_full_module_name
             except AttributeError:
-                type(_coconut_v).__module__ = _coconut_full_module_name
+                _coconut_v_type = type(_coconut_v)
+                if getattr(_coconut_v_type, "__module__", None) == str("__coconut__"):
+                    _coconut_v_type.__module__ = _coconut_full_module_name
     _coconut_sys.modules[_coconut_full_module_name] = _coconut__coconut__
 from __coconut__ import *
 from __coconut__ import _coconut_call_set_names, _coconut, _coconut_MatchError, _coconut_igetitem, _coconut_base_compose, _coconut_forward_compose, _coconut_back_compose, _coconut_forward_star_compose, _coconut_back_star_compose, _coconut_forward_dubstar_compose, _coconut_back_dubstar_compose, _coconut_pipe, _coconut_star_pipe, _coconut_dubstar_pipe, _coconut_back_pipe, _coconut_back_star_pipe, _coconut_back_dubstar_pipe, _coconut_none_pipe, _coconut_none_star_pipe, _coconut_none_dubstar_pipe, _coconut_bool_and, _coconut_bool_or, _coconut_none_coalesce, _coconut_minus, _coconut_map, _coconut_partial, _coconut_get_function_match_error, _coconut_base_pattern_func, _coconut_addpattern, _coconut_sentinel, _coconut_assert, _coconut_mark_as_match, _coconut_reiterable
 _coconut_sys.path.pop(0)
+
 # Compiled Coconut: -----------------------------------------------------------
 
 
 
+import random
+
 if _coconut_sys.version_info < (3, 3):
     from collections import Iterable
 else:
     from collections.abc import Iterable
 
 from bbopt import constants
+from bbopt.params import param_processor
 from bbopt.util import sorted_items
 from bbopt.util import convert_match_errors
-from bbopt.params import param_processor
+from bbopt.util import DictProxy
+from bbopt.util import ListProxy
+from bbopt.util import mean
 from bbopt.registry import backend_registry
 from bbopt.registry import alg_registry
 from bbopt.registry import meta_registry
@@ -64,14 +72,16 @@ def _init_backend(*_coconut_match_args, **_coconut_match_kwargs):
         _coconut_match_temp_2 = _coconut_match_args[2] if _coconut.len(_coconut_match_args) > 2 else _coconut_match_kwargs.pop("params")
         args = _coconut_match_args[3:]
         _coconut_match_temp_3 = _coconut_match_kwargs.pop("_attempt_to_update_backend") if "_attempt_to_update_backend" in _coconut_match_kwargs else None
+        _coconut_match_temp_4 = _coconut_match_kwargs.pop("_on_new_backend") if "_on_new_backend" in _coconut_match_kwargs else None
         backend_cls = _coconut_match_temp_0
         examples = _coconut_match_temp_1
         params = _coconut_match_temp_2
         _attempt_to_update_backend = _coconut_match_temp_3
+        _on_new_backend = _coconut_match_temp_4
         options = _coconut_match_kwargs
         _coconut_match_check_0 = True
     if not _coconut_match_check_0:
-        raise _coconut_FunctionMatchError('match def _init_backend(backend_cls, examples, params, *args, _attempt_to_update_backend=None, **options):', _coconut_match_args)
+        raise _coconut_FunctionMatchError('match def _init_backend(backend_cls, examples, params, *args, _attempt_to_update_backend=None, _on_new_backend=None, **options):', _coconut_match_args)
 
     backend_examples = examples[:]
     backend_params = params.copy()
@@ -89,20 +99,22 @@ def _init_backend(*_coconut_match_args, **_coconut_match_kwargs):
     if new_backend is None:
         assert not _attempt_to_update_backend or isinstance(_attempt_to_update_backend, Backend), "invalid backend to attempt update on: {_coconut_format_0}".format(_coconut_format_0=(_attempt_to_update_backend))
         new_backend = backend_cls(backend_examples, backend_params, *args, **options)
+        if _on_new_backend is not None:
+            _on_new_backend(new_backend)
 
     return new_backend
 
 
 def _make_safe_backend_store(backend_store, remove_backends):
     """Get a new backend_store without the given remove_backends."""
-    safe_backend_store = backend_store.copy()  # preserves the defaultdict
+    safe_backend_store = DictProxy(old_dict=backend_store, new_dict=backend_store.copy())
     for backend_cls in backend_store:
         if any((isinstance(rem_backend, backend_cls) for rem_backend in remove_backends)):
-            safe_specific_backend_store = []
+            safe_specific_backends = []
             for stored_args, stored_options, stored_backend in backend_store[backend_cls]:
                 if stored_backend not in remove_backends:
-                    safe_specific_backend_store.append((stored_args, stored_options, stored_backend))
-            safe_backend_store[backend_cls] = safe_specific_backend_store
+                    safe_specific_backends.append((stored_args, stored_options, stored_backend))
+            safe_backend_store[backend_cls] = ListProxy(old_list=backend_store[backend_cls], new_list=safe_specific_backends)
     return safe_backend_store
 
 
@@ -118,15 +130,17 @@ def get_backend(*_coconut_match_args, **_coconut_match_kwargs):
         _coconut_match_temp_3 = _coconut_match_args[3] if _coconut.len(_coconut_match_args) > 3 else _coconut_match_kwargs.pop("params")
         args = _coconut_match_args[4:]
         _coconut_match_temp_4 = _coconut_match_kwargs.pop("_current_backend") if "_current_backend" in _coconut_match_kwargs else None
+        _coconut_match_temp_5 = _coconut_match_kwargs.pop("_on_new_backend") if "_on_new_backend" in _coconut_match_kwargs else None
         backend_store = _coconut_match_temp_0
         backend = _coconut_match_temp_1
         examples = _coconut_match_temp_2
         params = _coconut_match_temp_3
         _current_backend = _coconut_match_temp_4
+        _on_new_backend = _coconut_match_temp_5
         options = _coconut_match_kwargs
         _coconut_match_check_1 = True
     if not _coconut_match_check_1:
-        raise _coconut_FunctionMatchError('match def get_backend(backend_store, backend, examples, params, *args, _current_backend=None, **options):', _coconut_match_args)
+        raise _coconut_FunctionMatchError('match def get_backend(backend_store, backend, examples, params, *args, _current_backend=None, _on_new_backend=None, **options):', _coconut_match_args)
 
     if isinstance(backend, type) and issubclass(backend, Backend):
         backend_cls = backend
@@ -142,10 +156,13 @@ def get_backend(*_coconut_match_args, **_coconut_match_kwargs):
             store_ind = i
             break
 
-    if backend_cls.request_backend_store is True:
-        options["backend_store"] = _make_safe_backend_store(backend_store, (attempt_to_update_backend, _current_backend))
+    if backend_cls.request_backend_store:
+        init_options = options.copy()
+        init_options["_backend_store"] = _make_safe_backend_store(backend_store, (attempt_to_update_backend,))
+    else:
+        init_options = options
 
-    new_backend = _init_backend(backend_cls, examples, params, *args, _attempt_to_update_backend=attempt_to_update_backend, **options)
+    new_backend = _init_backend(backend_cls, examples, params, *args, _attempt_to_update_backend=attempt_to_update_backend, _on_new_backend=_on_new_backend, **init_options)
 
     if store_ind is None:
         backend_store[backend_cls].append((args, options, new_backend))
@@ -274,6 +291,50 @@ def get_named_data_points_and_losses(examples, params, *args, **kwargs):
     return named_data_points, losses
 
 
+def marginalize(named_data_points, losses, param_name, ave_func=mean):
+    """Get an average loss for each prior value of param_name."""
+    losses_for_vals = []  # we can't use a dict since vals might not be hashable
+    for point, loss in zip(named_data_points, losses):
+        val = point[param_name]
+        for check_val, check_losses in losses_for_vals:
+            if check_val == val:
+                check_losses.append(loss)
+                break
+        else:  # no break
+            losses_for_vals.append((val, [loss]))
+
+    marginals = []
+    for val, all_losses in losses_for_vals:
+        marginals.append((val, ave_func(all_losses)))
+    return marginals
+
+
+def get_cum_probs_for(distribution):
+    """Generate cumulative probabilities from the given distribution."""
+    cum_probs = []
+    total_weight = sum((weight for elem, weight in distribution))
+    prev_cutoff = 0
+    for elem, weight in distribution:
+        if weight == float("inf"):
+            cutoff = 1
+        elif weight in (float("-inf"), float("nan")) or total_weight == float("nan"):
+            cutoff = prev_cutoff
+        else:
+            cutoff = prev_cutoff + weight / total_weight
+        cum_probs.append((elem, cutoff))
+        prev_cutoff = cutoff
+    return cum_probs
+
+
+def random_from_cum_probs(cum_probs):
+    """Randomly choose an element using cum_probs."""
+    rand_val = random.random()
+    for elem, cutoff in cum_probs:
+        if rand_val <= cutoff:
+            return elem
+    return None
+
+
 def make_values(params, point):
     """Return a dictionary with the values replaced by the values in point,
     where point is a list of the values corresponding to the sorted params."""
@@ -349,21 +410,23 @@ class Backend(_coconut.object):
 #  to allow fast updating on new data
     tell_examples = None
 
-# derived classes can set this to True to have a backend_store keyword
+# derived classes can set this to True to have a _backend_store keyword
 #  argument passed to __init__ with an object usable in get_backend
     request_backend_store = False
 
     def __new__(cls, examples=None, params=None, *args, **kwargs):
         self = super(Backend, cls).__new__(cls)
         if self.tell_examples is not None:
-            self._examples = examples
+            self._examples = []
             self._params = params
             self._args = args
             self._kwargs = kwargs
         return self
 
-    def __init__(self, examples=None, params=None):
-        pass
+    def __init__(self, examples=None, params=None, *args, **kwargs):
+        """Just call attempt_update by default."""
+        result = self.attempt_update(examples, params, *args, **kwargs)
+        assert result, "Backend.__init__: {_coconut_format_0}.attempt_update(*{_coconut_format_1}, **{_coconut_format_2}) failed with result {_coconut_format_3!r}".format(_coconut_format_0=(self.__class__.__name__), _coconut_format_1=(args), _coconut_format_2=(kwargs), _coconut_format_3=(result))
 
     def attempt_update(self, examples=None, params=None, *args, **kwargs):
         """Attempt to update this backend with new arguments. False indicates that the
@@ -378,6 +441,7 @@ class Backend(_coconut.object):
                 self.tell_examples(new_examples)
             except NotImplementedError:
                 return False
+        self._examples = examples
         return True
 
     def init_fallback_backend(self):
@@ -428,11 +492,8 @@ class Backend(_coconut.object):
         cls.register_meta(alg_name, cls.registered_algs, meta_alg)
 
     @staticmethod
-    def register_meta(alg_name, algs, meta_alg=None):
+    def register_meta(alg_name, algs, meta_alg=constants.default_alg_sentinel):
         """Register an algorithm that defers to run_meta."""
-        if meta_alg is None:
-            from bbopt.optimizer import BlackBoxOptimizer
-            meta_alg = BlackBoxOptimizer.DEFAULT_ALG_SENTINEL
         meta_registry.register(alg_name, (algs, meta_alg))
 
     @staticmethod
@@ -446,6 +507,7 @@ class StandardBackend(Backend):
     """Base class for standard BBopt backends."""
 
     def __init__(self, examples, params, *args, **kwargs):
+        """Implement __init__ using setup_backend and tell_examples."""
         self.init_fallback_backend()
 
         if not params:
