@@ -33,11 +33,13 @@ def main(num_trials=10):
     for i in range(num_trials):
         run_trial(serving=False)
     # Return the loss of the best run
-    return bb.get_optimal_run()["loss"]
+    return bb.get_best_run()["loss"]
 
 
 if __name__ == "__main__":
     print(main())
+    # check that we only ever created one backend
+    assert all(count <= 2 for count in bb._backend_creation_counts.values()), bb._backend_creation_counts
 
 
 best_y = run_trial(serving=True)
