@@ -1,8 +1,8 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
-# __coconut_hash__ = 0xc2778b94
+# __coconut_hash__ = 0x294515c0
 
-# Compiled with Coconut version 1.5.0-post_dev57 [Fish License]
+# Compiled with Coconut version 1.5.0-post_dev74 [Fish License]
 
 """
 The main BBopt interface.
@@ -27,11 +27,14 @@ if _coconut_module_name and _coconut_module_name[0].isalpha() and all(c.isalpha(
             try:
                 _coconut_v.__module__ = _coconut_full_module_name
             except AttributeError:
-                type(_coconut_v).__module__ = _coconut_full_module_name
+                _coconut_v_type = type(_coconut_v)
+                if getattr(_coconut_v_type, "__module__", None) == str("__coconut__"):
+                    _coconut_v_type.__module__ = _coconut_full_module_name
     _coconut_sys.modules[_coconut_full_module_name] = _coconut__coconut__
 from __coconut__ import *
 from __coconut__ import _coconut_call_set_names, _coconut, _coconut_MatchError, _coconut_igetitem, _coconut_base_compose, _coconut_forward_compose, _coconut_back_compose, _coconut_forward_star_compose, _coconut_back_star_compose, _coconut_forward_dubstar_compose, _coconut_back_dubstar_compose, _coconut_pipe, _coconut_star_pipe, _coconut_dubstar_pipe, _coconut_back_pipe, _coconut_back_star_pipe, _coconut_back_dubstar_pipe, _coconut_none_pipe, _coconut_none_star_pipe, _coconut_none_dubstar_pipe, _coconut_bool_and, _coconut_bool_or, _coconut_none_coalesce, _coconut_minus, _coconut_map, _coconut_partial, _coconut_get_function_match_error, _coconut_base_pattern_func, _coconut_addpattern, _coconut_sentinel, _coconut_assert, _coconut_mark_as_match, _coconut_reiterable
 _coconut_sys.path.pop(0)
+
 # Compiled Coconut: -----------------------------------------------------------
 
 
@@ -52,7 +55,6 @@ from pprint import pprint
 import numpy as np
 
 from bbopt import constants
-from bbopt.registry import backend_registry
 from bbopt.registry import alg_registry
 from bbopt.registry import meta_registry
 from bbopt.util import Str
@@ -93,15 +95,12 @@ def array_param(func, name, shape, kwargs):
 
 class BlackBoxOptimizer(_coconut.object):
     """Main bbopt optimizer object. See https://github.com/evhub/bbopt for documentation."""
-    DEFAULT_ALG_SENTINEL = object()
     backend = None
     _new_params = None
     _current_example = None
 
-    @convert_match_errors
     @_coconut_mark_as_match
     def __init__(*_coconut_match_args, **_coconut_match_kwargs):
-        """Construct a new BlackBoxOptimizer. It is recommended to pass file=__file__."""
         _coconut_match_check_0 = False
         _coconut_FunctionMatchError = _coconut_get_function_match_error()
         if (_coconut.len(_coconut_match_args) <= 2) and (_coconut.sum((_coconut.len(_coconut_match_args) > 0, "self" in _coconut_match_kwargs)) == 1) and (_coconut.sum((_coconut.len(_coconut_match_args) > 1, "file" in _coconut_match_kwargs)) == 1):
@@ -118,6 +117,8 @@ class BlackBoxOptimizer(_coconut.object):
         if not _coconut_match_check_0:
             raise _coconut_FunctionMatchError('match def __init__(self, file is Str, *, tag=None, protocol=None):', _coconut_match_args)
 
+        self._backend_creation_counts = defaultdict(int)
+
         self._file = norm_path(file)
         self._tag = tag
 
@@ -131,6 +132,31 @@ class BlackBoxOptimizer(_coconut.object):
 
         self.reload()
 
+    @convert_match_errors
+    @_coconut_addpattern(__init__)
+    @_coconut_mark_as_match
+    def __init__(*_coconut_match_args, **_coconut_match_kwargs):
+        """
+        Construct a new BlackBoxOptimizer. You must either pass file=__file__ or
+        both data_dir="/path/to/some/dir" and data_name="my_project_name".
+        """
+        _coconut_match_check_1 = False
+        _coconut_FunctionMatchError = _coconut_get_function_match_error()
+        if (_coconut.len(_coconut_match_args) <= 3) and (_coconut.sum((_coconut.len(_coconut_match_args) > 0, "self" in _coconut_match_kwargs)) == 1) and (_coconut.sum((_coconut.len(_coconut_match_args) > 1, "data_dir" in _coconut_match_kwargs)) == 1) and (_coconut.sum((_coconut.len(_coconut_match_args) > 2, "data_name" in _coconut_match_kwargs)) == 1):
+            _coconut_match_temp_0 = _coconut_match_args[0] if _coconut.len(_coconut_match_args) > 0 else _coconut_match_kwargs.pop("self")
+            _coconut_match_temp_1 = _coconut_match_args[1] if _coconut.len(_coconut_match_args) > 1 else _coconut_match_kwargs.pop("data_dir")
+            _coconut_match_temp_2 = _coconut_match_args[2] if _coconut.len(_coconut_match_args) > 2 else _coconut_match_kwargs.pop("data_name")
+            if (_coconut.isinstance(_coconut_match_temp_1, Str)) and (_coconut.isinstance(_coconut_match_temp_2, Str)):
+                self = _coconut_match_temp_0
+                data_dir = _coconut_match_temp_1
+                data_name = _coconut_match_temp_2
+                kwargs = _coconut_match_kwargs
+                _coconut_match_check_1 = True
+        if not _coconut_match_check_1:
+            raise _coconut_FunctionMatchError('addpattern def __init__(self, data_dir is Str, data_name is Str, **kwargs):', _coconut_match_args)
+
+        self.__init__(os.path.join(data_dir, data_name), **kwargs)
+
 # Private utilities:
 
     def _loads(self, raw_contents):
@@ -181,15 +207,15 @@ class BlackBoxOptimizer(_coconut.object):
         contents = df.read()
         if contents:
             _coconut_match_to_0 = self._loads(contents)
-            _coconut_match_check_1 = False
+            _coconut_match_check_2 = False
             if (_coconut.isinstance(_coconut_match_to_0, _coconut.abc.Mapping)) and (_coconut.len(_coconut_match_to_0) == 2):
                 _coconut_match_temp_0 = _coconut_match_to_0.get("params", _coconut_sentinel)
                 _coconut_match_temp_1 = _coconut_match_to_0.get("examples", _coconut_sentinel)
                 if (_coconut_match_temp_0 is not _coconut_sentinel) and (_coconut_match_temp_1 is not _coconut_sentinel):
                     params = _coconut_match_temp_0
                     examples = _coconut_match_temp_1
-                    _coconut_match_check_1 = True
-            if not _coconut_match_check_1:
+                    _coconut_match_check_2 = True
+            if not _coconut_match_check_2:
                 raise _coconut_MatchError('{"params": params, "examples": examples} = self._loads(contents)', _coconut_match_to_0)
 
             self._old_params = params
@@ -219,7 +245,9 @@ class BlackBoxOptimizer(_coconut.object):
 
     def _get_backend(self, backend, *args, **options):
         """Get the given backend, attempting to load from stored backends."""
-        return get_backend(self._backend_store, backend, self._examples, self._old_params, *args, _current_backend=self.backend, **options)
+        def _coconut_lambda_0(backend):
+            self._backend_creation_counts[type(backend)] += 1
+        return get_backend(self._backend_store, backend, self._examples, self._old_params, *args, _current_backend=self.backend, _on_new_backend=(_coconut_lambda_0), **options)
 
     def _get_skopt_backend(self):
         """Get a scikit-optimize backend regardless of whether currently using one."""
@@ -281,15 +309,15 @@ class BlackBoxOptimizer(_coconut.object):
         kwargs = param_processor.standardize_kwargs(kwargs)
 
         _coconut_match_to_1 = self._old_params
-        _coconut_match_check_2 = False
+        _coconut_match_check_3 = False
         if _coconut.isinstance(_coconut_match_to_1, _coconut.abc.Mapping):
             _coconut_match_temp_0 = _coconut_match_to_1.get(name, _coconut_sentinel)
             if (_coconut_match_temp_0 is not _coconut_sentinel) and (_coconut.isinstance(_coconut_match_temp_0, _coconut.abc.Sequence)) and (_coconut.len(_coconut_match_temp_0) == 3):
                 old_func = _coconut_match_temp_0[0]
                 old_args = _coconut_match_temp_0[1]
                 old_kwargs = _coconut_match_temp_0[2]
-                _coconut_match_check_2 = True
-        if _coconut_match_check_2:
+                _coconut_match_check_3 = True
+        if _coconut_match_check_3:
             if (func, args) != (old_func, old_args):
                 printerr("BBopt Warning: detected change in parameter {_coconut_format_0} ({_coconut_format_1} != {_coconut_format_2}) (you may need to delete your old BBopt data)".format(_coconut_format_0=(name), _coconut_format_1=((func, args)), _coconut_format_2=((old_func, old_args))))
 
@@ -311,10 +339,10 @@ class BlackBoxOptimizer(_coconut.object):
         """All algorithms supported by run."""
         return alg_registry.asdict()
 
-    def run(self, alg=DEFAULT_ALG_SENTINEL):
+    def run(self, alg=constants.default_alg_sentinel):
         """Optimize parameters using the given algorithm
         (use .algs to get the list of valid algorithms)."""
-        if alg is self.DEFAULT_ALG_SENTINEL:
+        if alg is constants.default_alg_sentinel:
             alg = constants.default_alg
         if alg in meta_registry:
             algs, meta_alg = meta_registry[alg]
@@ -323,9 +351,9 @@ class BlackBoxOptimizer(_coconut.object):
             backend, options = alg_registry[alg]
             self.run_backend(backend, **options)
 
-    def run_meta(self, algs, meta_alg=DEFAULT_ALG_SENTINEL):
+    def run_meta(self, algs, meta_alg=constants.default_alg_sentinel):
         """Dynamically choose the best algorithm from the given set of algorithms."""
-        if meta_alg is self.DEFAULT_ALG_SENTINEL:
+        if meta_alg is constants.default_alg_sentinel:
             meta_alg = constants.default_meta_alg
         self.run(meta_alg)
         alg = self.choice(constants.meta_opt_alg_var, algs)
@@ -406,7 +434,7 @@ class BlackBoxOptimizer(_coconut.object):
 
     def partial_dependence(self, i_name, j_name=None, *args, **kwargs):
         """Calls skopt.plots.partial_dependence where i_name and j_name are parameter names."""
-        def _coconut_mock_8(self, i_name, j_name=None, *args, **kwargs): return self, i_name, j_name, args, kwargs
+        def _coconut_mock_9(self, i_name, j_name=None, *args, **kwargs): return self, i_name, j_name, args, kwargs
         while True:
             from skopt.plots import partial_dependence
             if not self._examples:
@@ -419,18 +447,18 @@ class BlackBoxOptimizer(_coconut.object):
             j = None if j_name is None else sorted_names.index(j_name)
 
             try:
-                _coconut_tre_check_0 = partial_dependence is _coconut_recursive_func_26
+                _coconut_tre_check_0 = partial_dependence is _coconut_recursive_func_27
             except _coconut.NameError:
                 _coconut_tre_check_0 = False
             if _coconut_tre_check_0:
-                self, i_name, j_name, args, kwargs = _coconut_mock_8(skopt_backend.space, skopt_backend.model, i, j, *args, **kwargs)
+                self, i_name, j_name, args, kwargs = _coconut_mock_9(skopt_backend.space, skopt_backend.model, i, j, *args, **kwargs)
                 continue
             else:
                 return partial_dependence(skopt_backend.space, skopt_backend.model, i, j, *args, **kwargs)
 
 
             return None
-    _coconut_recursive_func_26 = partial_dependence
+    _coconut_recursive_func_27 = partial_dependence
     def plot_partial_dependence_1D(self, i_name, ax=None, yscale=None, **kwargs):
         """Constructs a 1D partial dependence plot using self.partial_dependence."""
         xi, yi = self.partial_dependence(i_name, **kwargs)
@@ -444,51 +472,51 @@ class BlackBoxOptimizer(_coconut.object):
 
     def plot_evaluations(self, *args, **kwargs):
         """Calls skopt.plots.plot_evaluations."""
-        def _coconut_mock_10(self, *args, **kwargs): return self, args, kwargs
+        def _coconut_mock_11(self, *args, **kwargs): return self, args, kwargs
         while True:
             from skopt.plots import plot_evaluations
             try:
-                _coconut_tre_check_1 = plot_evaluations is _coconut_recursive_func_29
+                _coconut_tre_check_1 = plot_evaluations is _coconut_recursive_func_30
             except _coconut.NameError:
                 _coconut_tre_check_1 = False
             if _coconut_tre_check_1:
-                self, args, kwargs = _coconut_mock_10(self.get_skopt_result(), *args, **kwargs)
+                self, args, kwargs = _coconut_mock_11(self.get_skopt_result(), *args, **kwargs)
                 continue
             else:
                 return plot_evaluations(self.get_skopt_result(), *args, **kwargs)
 
 
             return None
-    _coconut_recursive_func_29 = plot_evaluations
+    _coconut_recursive_func_30 = plot_evaluations
     def plot_objective(self, *args, **kwargs):
         """Calls skopt.plots.plot_objective."""
-        def _coconut_mock_11(self, *args, **kwargs): return self, args, kwargs
+        def _coconut_mock_12(self, *args, **kwargs): return self, args, kwargs
         while True:
             from skopt.plots import plot_objective
             try:
-                _coconut_tre_check_2 = plot_objective is _coconut_recursive_func_30
+                _coconut_tre_check_2 = plot_objective is _coconut_recursive_func_31
             except _coconut.NameError:
                 _coconut_tre_check_2 = False
             if _coconut_tre_check_2:
-                self, args, kwargs = _coconut_mock_11(self.get_skopt_result(), *args, **kwargs)
+                self, args, kwargs = _coconut_mock_12(self.get_skopt_result(), *args, **kwargs)
                 continue
             else:
                 return plot_objective(self.get_skopt_result(), *args, **kwargs)
 
 
             return None
-    _coconut_recursive_func_30 = plot_objective
+    _coconut_recursive_func_31 = plot_objective
     def plot_regret(self, *args, **kwargs):
         """Calls skopt.plots.plot_regret."""
-        def _coconut_mock_12(self, *args, **kwargs): return self, args, kwargs
+        def _coconut_mock_13(self, *args, **kwargs): return self, args, kwargs
         while True:
             from skopt.plots import plot_regret
             try:
-                _coconut_tre_check_3 = plot_regret is _coconut_recursive_func_31
+                _coconut_tre_check_3 = plot_regret is _coconut_recursive_func_32
             except _coconut.NameError:
                 _coconut_tre_check_3 = False
             if _coconut_tre_check_3:
-                self, args, kwargs = _coconut_mock_12(self.get_skopt_result(), *args, **kwargs)
+                self, args, kwargs = _coconut_mock_13(self.get_skopt_result(), *args, **kwargs)
                 continue
             else:
                 return plot_regret(self.get_skopt_result(), *args, **kwargs)
@@ -497,7 +525,7 @@ class BlackBoxOptimizer(_coconut.object):
 # Base random functions:
 
             return None
-    _coconut_recursive_func_31 = plot_regret
+    _coconut_recursive_func_32 = plot_regret
     def randrange(self, name, *args, **kwargs):
         """Create a new parameter with the given name modeled by random.randrange(*args)."""
         return self.param(name, "randrange", *args, **kwargs)
@@ -600,10 +628,10 @@ class BlackBoxOptimizer(_coconut.object):
             if len(sampling_population) <= 1:
                 sample.append(sampling_population[0])
             else:
-                def _coconut_lambda_0(val):
+                def _coconut_lambda_1(val):
                     elem = _coconut_igetitem(val, i)
                     return sampling_population.index(elem) if elem in sampling_population else 0
-                proc_kwargs = (param_processor.modify_kwargs)(_coconut_lambda_0, kwargs)
+                proc_kwargs = (param_processor.modify_kwargs)(_coconut_lambda_1, kwargs)
                 ind = self.randrange("{_coconut_format_0}[{_coconut_format_1}]".format(_coconut_format_0=(name), _coconut_format_1=(i)), len(sampling_population), **proc_kwargs)
                 sample.append(sampling_population.pop(ind))
         return sample
