@@ -1,8 +1,8 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
-# __coconut_hash__ = 0x20909e30
+# __coconut_hash__ = 0x1d5df9dd
 
-# Compiled with Coconut version 1.5.0-post_dev57 [Fish License]
+# Compiled with Coconut version 1.5.0-post_dev74 [Fish License]
 
 # Coconut Header: -------------------------------------------------------------
 
@@ -23,11 +23,14 @@ if _coconut_module_name and _coconut_module_name[0].isalpha() and all(c.isalpha(
             try:
                 _coconut_v.__module__ = _coconut_full_module_name
             except AttributeError:
-                type(_coconut_v).__module__ = _coconut_full_module_name
+                _coconut_v_type = type(_coconut_v)
+                if getattr(_coconut_v_type, "__module__", None) == str("__coconut__"):
+                    _coconut_v_type.__module__ = _coconut_full_module_name
     _coconut_sys.modules[_coconut_full_module_name] = _coconut__coconut__
 from __coconut__ import *
 from __coconut__ import _coconut_call_set_names, _coconut, _coconut_MatchError, _coconut_igetitem, _coconut_base_compose, _coconut_forward_compose, _coconut_back_compose, _coconut_forward_star_compose, _coconut_back_star_compose, _coconut_forward_dubstar_compose, _coconut_back_dubstar_compose, _coconut_pipe, _coconut_star_pipe, _coconut_dubstar_pipe, _coconut_back_pipe, _coconut_back_star_pipe, _coconut_back_dubstar_pipe, _coconut_none_pipe, _coconut_none_star_pipe, _coconut_none_dubstar_pipe, _coconut_bool_and, _coconut_bool_or, _coconut_none_coalesce, _coconut_minus, _coconut_map, _coconut_partial, _coconut_get_function_match_error, _coconut_base_pattern_func, _coconut_addpattern, _coconut_sentinel, _coconut_assert, _coconut_mark_as_match, _coconut_reiterable
 _coconut_sys.path.pop(0)
+
 # Compiled Coconut: -----------------------------------------------------------
 
 # Imports:
@@ -45,6 +48,9 @@ else:
 
 from coconut.command.util import call_output
 
+from bbopt.util import mean
+from bbopt.util import median
+
 
 # Constants:
 
@@ -89,9 +95,6 @@ mixture_data = os.path.join(example_dir, "mixture_example.bbopt.pickle")
 json_file = os.path.join(example_dir, "json_example.py")
 json_data = os.path.join(example_dir, "json_example.bbopt.json")
 
-remove_erroring_algs_file = os.path.join(example_dir, "remove_erroring_algs_example.py")
-remove_erroring_algs_data = os.path.join(example_dir, "remove_erroring_algs_example.bbopt.pickle")
-
 
 # Utilities:
 
@@ -146,17 +149,19 @@ def get_nums(inputstr, numtype=float):
             pass
 
 
-def mean(xs):
-    return sum(xs) / len(xs)
-def median(xs):
-    sorted_xs = (list)((sorted)(xs))
-    return mean((sorted_xs[len(sorted_xs) // 2], sorted_xs[(len(sorted_xs) + 1) // 2],))
 def middle_mean(xs):
+    """Mean of the middle half of xs."""
     a, b = len(xs) // 4, 3 * len(xs) // 4
     return mean(xs[a:b])
 
 
-def assert_improving(data, ave_func=mean):
+def stdev(xs):
+    """Standard deviation of xs."""
+    mu = mean(xs)
+
+
+    return mean(((x - mu)**2 for x in xs))**0.5
+def assert_improving(data, ave_func=mean, within_stdevs=0.5):
     """Assert that the second half of data is greater/smaller than the first."""
     examples = data["examples"]
     assert len(examples) >= 2, data
@@ -164,14 +169,22 @@ def assert_improving(data, ave_func=mean):
     first_half, second_half = examples[:half_pt], examples[half_pt:]
     if "loss" in first_half[0]:
         ave_func = min if ave_func is None else ave_func
-        first_losses = (ave_func)((map)(_coconut.operator.itemgetter(("loss")), first_half))
-        second_losses = (ave_func)((map)(_coconut.operator.itemgetter(("loss")), second_half))
-        assert second_losses < first_losses
+        first_losses = (map)(_coconut.operator.itemgetter(("loss")), first_half)
+        second_losses = (map)(_coconut.operator.itemgetter(("loss")), second_half)
+
+        first_ave_loss = ave_func(first_losses)
+        second_ave_loss = ave_func(second_losses)
+        first_stdev = stdev(first_losses)
+        assert second_ave_loss - first_ave_loss < first_stdev * within_stdevs
     else:
         ave_func = max if ave_func is None else ave_func
-        first_gains = (ave_func)((map)(_coconut.operator.itemgetter(("gain")), first_half))
-        second_gains = (ave_func)((map)(_coconut.operator.itemgetter(("gain")), second_half))
-        assert second_gains > first_gains
+        first_gains = (map)(_coconut.operator.itemgetter(("gain")), first_half)
+        second_gains = (map)(_coconut.operator.itemgetter(("gain")), second_half)
+
+        first_ave_gain = ave_func(first_gains)
+        second_ave_gain = ave_func(second_gains)
+        first_stdev = stdev(first_gains)
+        assert second_ave_gain - first_ave_gain > -first_stdev * within_stdevs
 
 
 def call_bbopt(fpath, trials=NUM_TRIALS, procs=NUM_PROCS):
@@ -317,7 +330,7 @@ class TestExamples(unittest.TestCase):
             assert os.path.exists(any_fast_data)
 
             from bbopt.examples import any_fast_example
-            assert_improving(any_fast_example.bb.get_data(print_data=True))
+            assert_improving(any_fast_example.bb.get_data(print_data=True), ave_func=None)
             assert any_fast_example.u == want
             assert any_fast_example.u < 1
             assert any_fast_example.bb.num_examples == NUM_TRIALS
@@ -354,18 +367,6 @@ class TestExamples(unittest.TestCase):
             assert json_example.y == want
             assert json_example.bb.num_examples == NUM_TRIALS
 
-    def test_remove_erroring_algs(self):
-        print("\ntest remove_erroring_algs:")
-        with using(remove_erroring_algs_data):
-            results = call_bbopt(remove_erroring_algs_file)
-            want = min(get_nums(results, numtype=float))
-            assert os.path.exists(remove_erroring_algs_data)
-
-            from bbopt.examples import remove_erroring_algs_example
-            assert_improving(remove_erroring_algs_example.bb.get_data(print_data=True))
-            assert remove_erroring_algs_example.y == want
-            assert remove_erroring_algs_example.bb.num_examples == NUM_TRIALS
-
 
 _coconut_call_set_names(TestExamples)
 if __name__ == "__main__":
